// Copyright (C) 1999-2005
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __gridbase_h__
#define __gridbase_h__

#include <tk.h>

#include "vector.h"

class Widget;

class Attribute {
 public:
  enum Style {SOLID,DASH};
  enum Font {
    DEFAULT=0,TIMES,HELVETICA,SYMBOL,COURIER,
    DEFAULTBOLD=10,TIMESBOLD,HELVETICABOLD,SYMBOLBOLD,COURIERBOLD,
    DEFAULTITALIC=20,TIMESITALIC,HELVETICAITALIC,SYMBOLITALIC,COURIERITALIC
  };
  enum Colour {BLACK,WHITE,RED,GREEN,BLUE,CYAN,MAGENTA,YELLOW};

 private:
  Widget* parent;

  Style style_;
  float width_;

  Font font_;
  int size_;
  Tk_Font tkfont_;

  Colour colour_;
  char* colorName_;
  int color_;

 private:
  void initFont();

 public:
  Attribute(Widget*);
  ~Attribute();

  void setStyle(double);
  void setWidth(double);
  void setSize(double);
  void setFont(double);
  void setColour(double);

  Style style() {return style_;}
  float width() {return width_;}
  int size() {return size_;}
  Font font() {return font_;}
  Colour colour() {return colour_;}

  const char* colorName() {return colorName_;}
  Tk_Font tkfont() {return tkfont_;}
  int color() {return color_;}
};

class GridBase {
 public:
  enum RenderMode {X11, PS};

 protected:
  Widget* parent;

  char* option_;
  RenderMode renderMode;
  Attribute* line;
  Attribute* text;

  GC gc_;
  Pixmap pixmap_;
  int byteorder_;
  Matrix psmatrix_;
  int psmode_;

 protected:
  int matrixMap(void*, Matrix&, const char*);

 private:
  int x11Line(int n, const float* x, const float* y);
  int x11Text(const char*, float, float, const char*, float, float);

  int psLine(int n, const float* x, const float* y);
  int psText(const char*, float, float, const char*, float, float);

  void psColor(Attribute*);
  void psWidth(Attribute*);
  void psStyle(Attribute*);

  void drawString(const char*, Vector, const char*);
  void drawRotString(const char*, Vector, const char*, double);

  double calcTextAngle(double);

 public:  
  GridBase(Widget*, const char*);
  virtual ~GridBase();

  int gFlush() {return 1;}
  int gLine(int n, const float* x, const float* y)
    {return (renderMode==X11) ? x11Line(n,x,y) : psLine(n,x,y);}
  int gQch(float*, float*);
  int gMark(int, const float*, const float*, int) {return 1;}
  int gText(const char* txt, float x, float y, 
	     const char* just, float upx, float upy)
    {return (renderMode==X11) ? 
       x11Text(txt,x,y,just,upx,upy) : psText(txt,x,y,just,upx,upy);}
  int gTxExt(const char*, float, float, const char*, 
	      float, float, float*, float*);
  int gAttr(int attr, double value, double* old, int prim);
  int gScales(float *alpha, float *beta);
  int gCap(int, int);

  const char* option() {return option_;}
};

#endif
