#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ColorMenu {} {
    global ds9
    global menu

    set menu(color,invert) [msgcat::mc "Invert Colormap"]
    set menu(color,reset) [msgcat::mc "Reset Colormap"]
    set menu(color,params) [msgcat::mc "Colormap Parameters..."]

    menu $ds9(mb).color -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Colormap"
    $ds9(mb).color add separator
    $ds9(mb).color add checkbutton -label $menu(color,invert) \
	-variable colorbar(invert) -command InvertColorbar
    $ds9(mb).color add command -label $menu(color,reset) \
	-command ResetColormap
    $ds9(mb).color add separator
    $ds9(mb).color add command -label $menu(color,params) \
	-command ColormapDialog

    ColorPrefsMenu
}

proc ColorPrefsMenu {} {
    global ds9
    global menu
    global prefs

    menu $ds9(mb).prefs.color -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.color add separator
    $ds9(mb).prefs.color add checkbutton -label $menu(color,invert) \
	-variable prefs(colorbar,invert)
}

proc ColorButtons {} {
    global ds9
    global buttons
    global colorbar

    set ids [colorbar list id]

    frame $ds9(buttons).color1
    for {set i 0} {$i<9} {incr i} {
	set id [lindex $ids $i]
	set title [colorbar get name $id]
	set name [string tolower $title]
	RadioButton $ds9(buttons).color1.$i $name \
	    colorbar(map) $title "ChangeColormapID $id"
    }
    RadioButton $ds9(buttons).color1.more $buttons(more) \
	buttons(majorCurrent) $ds9(buttons).color2 MajorButton

    frame $ds9(buttons).color2
    for {set i 9} {$i<18} {incr i} {
	set id [lindex $ids $i]
	set title [colorbar get name $id]
	set name [string tolower $title]
	RadioButton $ds9(buttons).color2.$i $name \
	    colorbar(map) $title "ChangeColormapID $id"
    }
    RadioButton $ds9(buttons).color2.more $buttons(more) \
	buttons(majorCurrent) $ds9(buttons).color3 MajorButton

    frame $ds9(buttons).color3
    CheckButton $ds9(buttons).color3.invert invert \
	colorbar(invert) InvertColorbar
    ButtonButton $ds9(buttons).color3.reset reset ResetColormap
    RadioButton $ds9(buttons).color3.more $buttons(more) \
	buttons(majorCurrent) $ds9(buttons).color1 MajorButton
}

proc UpdateColorMenu {} {
    global ds9
    global current
    global menu
    global colorbar

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateColorMenu"
    }

    if {$current(frame) != {}} {
	if {[$current(frame) get type] == "rgb"} {
	    $ds9(mb).color entryconfig $menu(color,invert) -state disabled
	    for {set i 1} {$i<$colorbar(count)} {incr i} {
		$ds9(mb).color entryconfig $i -state disabled
	    }

	    for {set i 0} {$i<$colorbar(button1)} {incr i} {
		$ds9(buttons).color1.$i configure -state disabled
	    }
	    for {set i $colorbar(button1)} {$i<$colorbar(button2)} {incr i} {
		$ds9(buttons).color2.$i configure -state disabled
	    }
	    $ds9(buttons).color3.invert configure -state disabled

	} else {
	    $ds9(mb).color entryconfig $menu(color,invert) -state normal
	    for {set i 1} {$i<$colorbar(count)} {incr i} {
		$ds9(mb).color entryconfig $i -state normal
	    }

	    for {set i 0} {$i<$colorbar(button1)} {incr i} {
		$ds9(buttons).color1.$i configure -state normal
	    }
	    for {set i $colorbar(button1)} {$i<$colorbar(button2)} {incr i} {
		$ds9(buttons).color2.$i configure -state normal
	    }
	    $ds9(buttons).color3.invert configure -state normal
	}
    } else {
	$ds9(mb).color entryconfig $menu(color,invert) -state normal
	for {set i 1} {$i<$colorbar(count)} {incr i} {
	    $ds9(mb).color entryconfig $i -state normal
	}

	for {set i 0} {$i<$colorbar(button1)} {incr i} {
	    $ds9(buttons).color1.$i configure -state normal
	}
	for {set i $colorbar(button1)} {$i<$colorbar(button2)} {incr i} {
	    $ds9(buttons).color2.$i configure -state normal
	}
	$ds9(buttons).color3.invert configure -state normal
    }
}

