#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc FileMenu {} {
    global ds9
    global menu

    set menu(file,about) [msgcat::mc "About SAOImage DS9..."]
    set menu(file,openother) [msgcat::mc "Open Other"]
    set menu(file,preserve) [msgcat::mc "Preserve During Load"]
    set menu(file,saveimage) [msgcat::mc "Save Image as..."]
    set menu(file,savefits) [msgcat::mc "Save Frame as Fits..."]
    set menu(file,savempeg) [msgcat::mc "Save Frames as MPEG..."]
    set menu(file,header) [msgcat::mc "Display Fits Header..."]
    set menu(file,xpa) [msgcat::mc "XPA Information..."]
    set menu(file,source) [msgcat::mc "Source TCL..."]
    set menu(file,exit) [msgcat::mc "Exit"]

    menu $ds9(mb).file -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 File"
    $ds9(mb).file add command -label $menu(file,about) -command AboutBox
    $ds9(mb).file add separator
    $ds9(mb).file add command -label $menu(file,open) -command OpenFits
    $ds9(mb).file add cascade -label $menu(file,openother) \
	-menu $ds9(mb).file.open
    $ds9(mb).file add separator
    $ds9(mb).file add cascade -label $menu(file,preserve) \
	-menu $ds9(mb).file.preserve
    $ds9(mb).file add separator
    $ds9(mb).file add command -label $menu(file,saveimage) \
	-command SaveImageDialog
    $ds9(mb).file add command -label $menu(file,savefits) \
	-command SaveFitsDialog
    $ds9(mb).file add command -label $menu(file,savempeg) \
	-command SaveMPEGDialog
    $ds9(mb).file add separator
    $ds9(mb).file add command -label $menu(file,header) \
	-command DisplayHeaderMenu 
    $ds9(mb).file add separator
    $ds9(mb).file add command -label $menu(file,xpa) \
	-command DisplayXPAInfo
    $ds9(mb).file add command -label $menu(file,source) \
	-command OpenSource
    $ds9(mb).file add separator
    $ds9(mb).file add command -label $menu(file,print) -command Print
    $ds9(mb).file add command -label $menu(file,page) -command PageSetupDialog
    $ds9(mb).file add separator
    $ds9(mb).file add command -label $menu(file,exit) -command QuitDS9

    # File Open Menu

    set menu(file,open,array) [msgcat::mc "Open Array..."]
    set menu(file,open,url) [msgcat::mc "Open URL..."]
    set menu(file,open,rgb,fits,image) [msgcat::mc "Open RGB Fits Image..."]
    set menu(file,open,rgb,fits,cube) [msgcat::mc "Open RGB Fits Cube..."]
    set menu(file,open,rgb,array) [msgcat::mc "Open RGB Array..."]
    set menu(file,open,mosaic,datacube) \
	[msgcat::mc "Open Multi Ext as Data Cube..."]
    set menu(file,open,mosaic,iraf) [msgcat::mc "Open Mosaic IRAF..."]
    set menu(file,open,mosaic,iraf,seg) \
	[msgcat::mc "Open Mosaic IRAF Segment..."]
    set menu(file,open,mosaic,wcs) [msgcat::mc "Open Mosaic WCS..."]
    set menu(file,open,mosaic,wcs,next) [msgcat::mc "Open Mosaic WCS Next..."]
    set menu(file,open,mosaic,wcs,seg) \
	[msgcat::mc "Open Mosaic WCS Segment..."]
    set menu(file,open,mosaic,wfpc2) [msgcat::mc "Open Mosaic WFPC2..."]

    menu $ds9(mb).file.open -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 File Open"
    $ds9(mb).file.open add command -label $menu(file,open,array) \
	-command OpenArray
    $ds9(mb).file.open add command -label $menu(file,open,url) \
	-command OpenURL
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command -label $menu(file,open,rgb,fits,image) \
	-command OpenRGBImageFits
    $ds9(mb).file.open add command -label $menu(file,open,rgb,fits,cube) \
	-command OpenRGBCubeFits
    $ds9(mb).file.open add command -label $menu(file,open,rgb,array) \
	-command OpenRGBArray
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,datacube) \
	-command OpenDataCubeFits
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,iraf) \
	-command OpenMosaicImageIRAFFits
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,iraf,seg) \
	-command OpenMosaicIRAFFits
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,wcs) \
	-command OpenMosaicImageWCSFits
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,wcs,next) \
	-command OpenMosaicImageNextWCSFits
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,wcs,seg) \
	-command OpenMosaicWCSFits
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command -label $menu(file,open,mosaic,wfpc2) \
	-command OpenMosaicImageWFPC2Fits

    # File Preserve Menu

    set menu(file,preserve,scale) [msgcat::mc "Scale"]
    set menu(file,preserve,pan) [msgcat::mc "Pan"]
    set menu(file,preserve,regions) [msgcat::mc "Regions"]

    menu $ds9(mb).file.preserve -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 File Preserve"
    $ds9(mb).file.preserve add checkbutton \
	-label $menu(file,preserve,scale) \
	-variable scale(preserve) -command PreserveScale
    $ds9(mb).file.preserve add checkbutton \
	-label $menu(file,preserve,pan) \
	-variable panzoom(preserve) -command PreservePan
    $ds9(mb).file.preserve add checkbutton \
	-label $menu(file,preserve,regions) \
	-variable marker(preserve) -command MarkerPreserve

    FilePrefsMenu
}

proc FilePrefsMenu {} {
    global ds9
    global menu
    global prefs

    menu $ds9(mb).prefs.file -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.file add command -label $menu(file,print) \
	-command {PrefsPrintDialog ps}
    $ds9(mb).prefs.file add command -label $menu(file,page) \
	-command PrefsPageSetupDialog
}

proc FileButtons {} {
    global ds9
    global buttons

    set buttons(file,about) [msgcat::mc "about"]
    set buttons(file,open) [msgcat::mc "open"]
    set buttons(file,saveimage) [msgcat::mc "save img"]
    set buttons(file,savefits) [msgcat::mc "save fits"]
    set buttons(file,savempeg) [msgcat::mc "save mpeg"]
    set buttons(file,header) [msgcat::mc "header"]
    set buttons(file,source) [msgcat::mc "source"]
    set buttons(file,print) [msgcat::mc "print"]
    set buttons(file,page) [msgcat::mc "page"]
    set buttons(file,exit) [msgcat::mc "exit"]

    frame $ds9(buttons).file

    ButtonButton $ds9(buttons).file.about $buttons(file,about) AboutBox
    ButtonButton $ds9(buttons).file.open $buttons(file,open) OpenFits
    ButtonButton $ds9(buttons).file.saveimage $buttons(file,saveimage) \
	SaveImageDialog
    ButtonButton $ds9(buttons).file.savefits $buttons(file,savefits) \
	SaveFitsDialog
    ButtonButton $ds9(buttons).file.savempeg $buttons(file,savempeg) \
	SaveMPEGDialog
    ButtonButton $ds9(buttons).file.header $buttons(file,header) \
	DisplayHeaderMenu
    ButtonButton $ds9(buttons).file.src $buttons(file,source) OpenSource
    ButtonButton $ds9(buttons).file.print $buttons(file,print) Print
    ButtonButton $ds9(buttons).file.page $buttons(file,page) PageSetupDialog
    ButtonButton $ds9(buttons).file.exit $buttons(file,exit) QuitDS9
}

proc UpdateFileMenu {} {
    global ds9
    global current
    global crosshair
    global menu
 
    global debug
    if {$debug(tcl,update)} {
	puts "UpdateFileMenu"
    }

    if {$current(frame) != {}} {
	$ds9(mb).file entryconfig $menu(file,open) -state normal
	$ds9(mb).file entryconfig $menu(file,openother) -state normal
	$ds9(mb).file entryconfig $menu(file,saveimage) -state normal
	$ds9(mb).file entryconfig $menu(file,savefits) -state normal
	$ds9(mb).file entryconfig $menu(file,savempeg) -state normal
	$ds9(mb).file entryconfig $menu(file,header) -state disabled
	$ds9(mb).file entryconfig $menu(file,print) -state normal

	if {[$current(frame) get type] == "rgb"} {
	    $ds9(mb).file.open entryconfig $menu(file,open,rgb,fits,image) \
		-state normal
	    $ds9(mb).file.open entryconfig $menu(file,open,rgb,fits,cube) \
		-state normal
	    $ds9(mb).file.open entryconfig $menu(file,open,rgb,array) \
		-state normal
	} else {
	    $ds9(mb).file.open entryconfig $menu(file,open,rgb,fits,image) \
		-state disabled
	    $ds9(mb).file.open entryconfig $menu(file,open,rgb,fits,cube) \
		-state disabled
	    $ds9(mb).file.open entryconfig $menu(file,open,rgb,array) \
		-state disabled
	}

	$ds9(buttons).file.open configure -state normal
	$ds9(buttons).file.saveimage configure -state normal
	$ds9(buttons).file.savefits configure -state normal
	$ds9(buttons).file.savempeg configure -state normal
	$ds9(buttons).file.header configure -state disabled
	$ds9(buttons).file.print configure -state normal

	if {[$current(frame) has fits]} {
	    $ds9(mb).file entryconfig $menu(file,header) -state normal
	    $ds9(buttons).file.header configure -state normal
	}
    } else {
	$ds9(mb).file entryconfig $menu(file,open) -state disabled
	$ds9(mb).file entryconfig $menu(file,openother) -state disabled
	$ds9(mb).file entryconfig $menu(file,saveimage) -state disabled
	$ds9(mb).file entryconfig $menu(file,savefits) -state disabled
	$ds9(mb).file entryconfig $menu(file,savempeg) -state disabled
	$ds9(mb).file entryconfig $menu(file,header) -state disabled
	$ds9(mb).file entryconfig $menu(file,print) -state disabled

	$ds9(buttons).file.open configure -state disabled
	$ds9(buttons).file.saveimage configure -state disabled
	$ds9(buttons).file.savefits configure -state disabled
	$ds9(buttons).file.savempeg configure -state disabled
	$ds9(buttons).file.header configure -state disabled
	$ds9(buttons).file.print configure -state disabled
    }
}

