#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CenterCurrentFrame {} {
    global current

    CenterFrame $current(frame)
}

proc CenterAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	CenterFrame $f
    }
}

proc CenterFrame {which} {
    if {$which != {}} {
	SetWatchCursor
	$which center
	UpdateGraphXAxis
	UnsetWatchCursor

        UpdateZoomMenu
	UpdateZoomDialog
    }
}

proc Pan {x y sys sky} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) pan $sys $sky $x $y
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc PanTo {x y sys sky} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) pan to $sys $sky $x $y
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonPan {which x y} {
    global panzoom

    switch -- $panzoom(mode) {
	click {}
	drag {
	    $which pan motion begin $x $y
	}
	panzoom {}
    }
}

proc MotionPan {which x y} {
    global panzoom

    switch -- $panzoom(mode) {
	click {}
	drag {
	    $which pan motion $x $y
	}
	panzoom {}
    }
}

proc ReleasePan {which x y} {
    global panzoom
    global current

    switch -- $panzoom(mode) {
	click {
	    SetWatchCursor
	    $which pan to canvas $x $y
	}
	drag {
	    $which pan motion end $x $y

	    # we need to delay setting the cursor here, since set cursor causes
	    # an update, so the display will flash

	    SetWatchCursor
	}
	panzoom {
	    if {$panzoom(last) != " $x $y "} {
		set panzoom(state) 1
	    }
	    SetWatchCursor
	    switch -- $panzoom(state) {
		1 {
		    $which pan to canvas $x $y
		    $which update
		    set c [$which get cursor canvas]
		    $which warp to $c
		    set panzoom(last) $c

		    set z [$current(frame) get zoom]
		    if {$z < 2} {
			set panzoom(state) 2
		    } elseif {$z < 4} {
			set panzoom(state) 3
		    } elseif {$z < 8} {
			set panzoom(state) 4
		    } else {
			set panzoom(state) 5
		    }
		}
		2 {
		    $which zoom to 2 2 about canvas $x $y
		    set panzoom(state) 3
		}
		3 {
		    $which zoom to 4 4 about canvas $x $y
		    set panzoom(state) 4
		}
		4 {
		    $which zoom to 8 8 about canvas $x $y
		    set panzoom(state) 5
		}
		5 {
		    $which zoom to 1 1 about canvas $x $y
		    set panzoom(state) 2
		}
	    }
	    if {$which == $current(frame)} {
		set current(zoom) [$current(frame) get zoom]
	    }
	}
    }
    UpdateGraphXAxis
    UpdateZoomDialog
    UnsetWatchCursor
}

proc PreservePan {} {
    global current
    global panzoom

    if {$current(frame) != {}} {
	$current(frame) pan preserve $panzoom(preserve)
    }
}

proc ZoomToFit {} {
    global current
    global grid

    if {$current(frame) != {}} {
	SetWatchCursor

	# we need to update the grid because titles are zoom dependant
	if {$grid(view) && $grid(type) == "publication"} {
	    # recalculate to make room for labels
	    $current(frame) zoom to fit .8
	    set current(zoom) [$current(frame) get zoom]
	} else {
	    $current(frame) zoom to fit
	    set current(zoom) [$current(frame) get zoom]
	}
	GridUpdateZoom

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeZoom {} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) zoom to $current(zoom)

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc Zoom {x y} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) zoom $x $y
	set current(zoom) [$current(frame) get zoom]

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc ButtonZoom {which x y} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$which zoom 2 2 about canvas $x $y
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc ShiftZoom {which} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$which zoom .5 .5
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc ChangeOrient {} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) orient $current(orient)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc Rotate {value} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) rotate $value
	set current(rotate) [$current(frame) get rotate]

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeRotate {} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) rotate to $current(rotate)

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonRotate {which x y} {
    global panzoom

    $which rotate motion begin
    set panzoom(x) $x
}

proc MotionRotate {which x y} {
    global current
    global panzoom
    global cursor

    $which rotate motion [expr double($panzoom(x)-$x)/$panzoom(speed) * 180.]

    set current(rotate) [$which get rotate fixed]
    RefreshInfoBox $current(frame)
}

proc ReleaseRotate {which x y} {
    global current

    $which rotate motion end
    set current(rotate) [$which get rotate fixed]

    # we need to delay setting the cursor here, since set cursor causes
    # an update, so the display will flash

    SetWatchCursor

    RefreshInfoBox $which
    UpdateGraphXAxis
    UpdateZoomDialog

    UnsetWatchCursor
}

proc PanZoomDialog {} {
    global panzoom
    global panzoomdlg
    global menu
    global ds9

    # see if we already have a window visible
    if [winfo exist $panzoom(top)] {
	raise $panzoom(top)
	return
    }

    set w $panzoom(top)
    set title "Pan Zoom Rotate"

    # create the window
    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PanZoomDestroyDialog
    wm minsize $w 375 100

    # init variables
    set panzoomdlg(clabel) $panzoom(system)

    $w configure -menu $panzoom(mb)

    menu $panzoom(mb) -tearoff 0
    $panzoom(mb) add cascade -label $menu(file) -menu $panzoom(mb).file
    $panzoom(mb) add cascade -label $menu(edit) -menu $panzoom(mb).edit
    $panzoom(mb) add cascade -label Coordinate -menu $panzoom(mb).coord
    $panzoom(mb) add cascade -label Pan -menu $panzoom(mb).pan
    $panzoom(mb) add cascade -label Zoom -menu $panzoom(mb).zoom
    $panzoom(mb) add cascade -label Orient -menu $panzoom(mb).orient
    $panzoom(mb) add cascade -label Rotate -menu $panzoom(mb).rotate

    menu $panzoom(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).file add command -label $menu(file,apply) \
	-command PanZoomApplyDialog
    $panzoom(mb).file add separator
    $panzoom(mb).file add command -label $menu(file,close) \
	-command PanZoomDestroyDialog

    menu $panzoom(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).edit add command -label $menu(edit,cut) \
	-command PanZoomCutDialog
    $panzoom(mb).edit add command -label $menu(edit,copy) \
	-command PanZoomCopyDialog
    $panzoom(mb).edit add command -label $menu(edit,paste) \
	-command PanZoomPasteDialog

    menu $panzoom(mb).coord -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).coord add radiobutton -label $menu(wcs) \
	-variable panzoom(system) -value wcs -command PanZoomDialogCoord
    $panzoom(mb).coord add cascade -label $menu(wcs,multi) \
	-menu $panzoom(mb).coord.wcs
    $panzoom(mb).coord add separator
    $panzoom(mb).coord add radiobutton -label $menu(wcs,image) \
	-variable panzoom(system) -value image -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,physical) \
	-variable panzoom(system) -value physical -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,amplifier) \
	-variable panzoom(system) -value amplifier -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,detector) \
	-variable panzoom(system) -value detector -command PanZoomDialogCoord
    $panzoom(mb).coord add separator
    $panzoom(mb).coord add radiobutton -label $menu(wcs,fk4) \
	-variable panzoom(sky) -value fk4  -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,fk5) \
	-variable panzoom(sky) -value fk5  -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,icrs) \
	-variable panzoom(sky) -value icrs  -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,galactic) \
	-variable panzoom(sky) -value galactic  -command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,ecliptic) \
	-variable panzoom(sky) -value ecliptic  -command PanZoomDialogCoord
    $panzoom(mb).coord add separator
    $panzoom(mb).coord add radiobutton -label $menu(wcs,degrees) \
	-variable panzoom(skyformat) -value degrees \
	-command PanZoomDialogCoord
    $panzoom(mb).coord add radiobutton -label $menu(wcs,sexagesimal) \
	-variable panzoom(skyformat) -value sexagesimal  \
	-command PanZoomDialogCoord

    menu $panzoom(mb).coord.wcs -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$panzoom(mb).coord.wcs add radiobutton -label "$menu(wcs) $l" \
	    -variable panzoom(system) -value "wcs$l" \
	    -command PanZoomDialogCoord
    }

    menu $panzoom(mb).pan -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).pan add command -label $menu(zoom,center) \
	-command CenterCurrentFrame

    menu $panzoom(mb).zoom -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).zoom add checkbutton -label $menu(zoom,align) \
	-variable wcs(align) -command AlignWCSFrame
    $panzoom(mb).zoom add cascade -label $menu(zoom,align,wcs) \
	-menu $panzoom(mb).zoom.align
    $panzoom(mb).zoom add separator
    $panzoom(mb).zoom add command -label $menu(zoom,zoom,in) \
	-command {Zoom 2 2}
    $panzoom(mb).zoom add command -label $menu(zoom,zoom,out) \
	-command {Zoom .5 .5}
    $panzoom(mb).zoom add separator
    $panzoom(mb).zoom add command -label $menu(zoom,zoom,tofit) \
	-command ZoomToFit
    $panzoom(mb).zoom add separator
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,1/32) \
	-variable current(zoom) -value { 0.03125 0.03125 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,1/16) \
	-variable current(zoom) -value { 0.0625 0.0625 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,1/8) \
	-variable current(zoom) -value { 0.125 0.125 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,1/4) \
	-variable current(zoom) -value { 0.25 0.25 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,1/2) \
	-variable current(zoom) -value { 0.5 0.5 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,1) \
	-variable current(zoom) -value { 1 1 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,2) \
	-variable current(zoom) -value { 2 2 }  -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,4) \
	-variable current(zoom) -value { 4  4 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,8) \
	-variable current(zoom) -value { 8 8 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,16) \
	-variable current(zoom) -value { 16 16 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label $menu(zoom,zoom,32) \
	-variable current(zoom) -value { 32 32 } -command ChangeZoom

    menu $panzoom(mb).zoom.align -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) 
    $panzoom(mb).zoom.align add radiobutton -label $menu(wcs) \
	-variable wcs(align,system) -value wcs -command AlignWCSFrame
    $panzoom(mb).zoom.align add cascade -label $menu(wcs,multi) \
	-menu $panzoom(mb).zoom.align.wcs
    $panzoom(mb).zoom.align add separator
    $panzoom(mb).zoom.align add radiobutton -label $menu(wcs,fk4) \
	-variable wcs(align,sky) -value fk4 -command AlignWCSFrame
    $panzoom(mb).zoom.align add radiobutton -label $menu(wcs,fk5) \
	-variable wcs(align,sky) -value fk5 -command AlignWCSFrame
    $panzoom(mb).zoom.align add radiobutton -label $menu(wcs,icrs) \
	-variable wcs(align,sky) -value icrs -command AlignWCSFrame
    $panzoom(mb).zoom.align add radiobutton -label $menu(wcs,galactic) \
	-variable wcs(align,sky) -value galactic -command AlignWCSFrame
    $panzoom(mb).zoom.align add radiobutton -label $menu(wcs,ecliptic) \
	-variable wcs(align,sky) -value ecliptic -command AlignWCSFrame

    menu $panzoom(mb).zoom.align.wcs -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$panzoom(mb).zoom.align.wcs add radiobutton -label "$menu(wcs) $l" \
	    -variable wcs(align,system) -value "wcs$l" -command AlignWCSFrame
    }

    menu $panzoom(mb).orient -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).orient add radiobutton  -label $menu(zoom,invert,none) \
	-variable current(orient) -value none -command ChangeOrient
    $panzoom(mb).orient add radiobutton  -label $menu(zoom,invert,x) \
	-variable current(orient) -value x -command ChangeOrient
    $panzoom(mb).orient add radiobutton -label $menu(zoom,invert,y) \
	-variable current(orient) -value y -command ChangeOrient
    $panzoom(mb).orient add radiobutton -label $menu(zoom,invert,xy) \
	-variable current(orient) -value xy -command ChangeOrient

    menu $panzoom(mb).rotate -tearoff 0 -selectcolor $menu(selectcolor)
    $panzoom(mb).rotate add radiobutton -label $menu(zoom,rot,0) \
	-variable current(rotate) -value 0 -command ChangeRotate
    $panzoom(mb).rotate add radiobutton -label $menu(zoom,rot,90) \
	-variable current(rotate) -value 90 -command ChangeRotate
    $panzoom(mb).rotate add radiobutton -label $menu(zoom,rot,180) \
	-variable current(rotate) -value 180 -command ChangeRotate
    $panzoom(mb).rotate add radiobutton -label $menu(zoom,rot,270) \
	-variable current(rotate) -value 270 -command ChangeRotate

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.zoomtitle -text "Zoom"
    entry $w.param.zoomx -textvariable panzoomdlg(zoom,x) -width 14
    entry $w.param.zoomy -textvariable panzoomdlg(zoom,y) -width 14

    label $w.param.rottitle -text "Rotate"
    entry $w.param.rotvalue -textvariable panzoomdlg(rotate) -width 14
    label $w.param.rottitle2 -text "degrees"

    label $w.param.pantitle -text "Pan"
    set panzoomdlg(xname) [label $w.param.panxtitle -text "" -width 1]
    entry $w.param.panx -textvariable panzoomdlg(x) -width 14
    set panzoomdlg(yname) [label $w.param.panytitle -text "" -width 1]
    entry $w.param.pany -textvariable panzoomdlg(y) -width 14
    label $w.param.pansystem -textvariable panzoomdlg(clabel) \
	-width 10 -relief groove
    
    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4

    grid $w.param.zoomtitle - $w.param.zoomx  - $w.param.zoomy \
	-padx 4 -sticky w
    grid $w.param.rottitle - $w.param.rotvalue $w.param.rottitle2 - \
	-padx 4 -sticky w
    grid $w.param.pantitle $panzoomdlg(xname) \
	$w.param.panx $panzoomdlg(yname) \
	$w.param.pany $w.param.pansystem -padx 4 -sticky w

    button $w.buttons.apply -text $menu(file,apply) \
	-command PanZoomApplyDialog
    button $w.buttons.close -text $menu(file,close) \
	-command PanZoomDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    $w.param.zoomx select range 0 end

    UpdateZoomDialog
}

proc PanZoomApplyDialog {} {
    global panzoom
    global panzoomdlg
    global current

    if {$current(frame) != {}} {
	set current(zoom) "$panzoomdlg(zoom,x) $panzoomdlg(zoom,y)"
	set current(rotate) $panzoomdlg(rotate)

	SetWatchCursor
	$current(frame) zoom to $current(zoom) about \
	    $panzoom(system) $panzoom(sky) $panzoomdlg(x) $panzoomdlg(y) 
	$current(frame) rotate to $current(rotate)

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	GridUpdateZoom
	UnsetWatchCursor

	UpdateZoomMenu
	UpdateZoomDialog
    }
}

proc PanZoomDestroyDialog {} {
    global panzoom
    global panzoomdlg

    if {[winfo exist $panzoom(top)]} {
	destroy $panzoom(top)
	destroy $panzoom(mb)
    }

    if {[info exist panzoomdlg]} {
	unset panzoomdlg
    }
}

proc PanZoomCutDialog {} {
    global panzoom

    tk_entryCut [focus -displayof $panzoom(top)]
}

proc PanZoomCopyDialog {} {
    global panzoom

    tk_entryCopy [focus -displayof $panzoom(top)]
}

proc PanZoomPasteDialog {} {
    global panzoom

    tk_entryPaste [focus -displayof $panzoom(top)]
}

proc PanZoomDialogCoord {} {
    global panzoom
    global panzoomdlg
    global current

    UpdateCoordMenu $current(frame) $panzoom(mb).coord
    AdjustCoord $current(frame) panzoom(system)

    if {$current(frame) != {}} {
	set coord [$current(frame) get cursor $panzoom(system) $panzoom(sky) \
		       $panzoom(skyformat)]
	set panzoomdlg(x) [lindex $coord 0]
	set panzoomdlg(y) [lindex $coord 1]
    } else {
	set panzoomdlg(x) {}
	set panzoomdlg(y) {}
    }

    $panzoomdlg(xname) configure -text "X" -font {helvetica 9}
    $panzoomdlg(yname) configure -text "Y" -font {helvetica 9}
    switch -- $panzoom(system) {
	image -
	physical -
	amplifier -
	detector {}
	default {
	    if [$current(frame) has wcs equatorial $panzoom(system)] {
		switch -- $panzoom(sky) {
		    fk4 -
		    fk5 -
		    icrs {
			$panzoomdlg(xname) configure -text "a" -font {symbol 12}
			$panzoomdlg(yname) configure -text "d" -font {symbol 12}
		    }
		    galactic {
			$panzoomdlg(xname) configure -text "l" \
			    -font {-family times -slant italic}
			$panzoomdlg(yname) configure -text "b" \
			    -font {-family times -slant italic}
		    }
		    ecliptic {
			$panzoomdlg(xname) configure -text "l" -font {symbol 12}
			$panzoomdlg(yname) configure -text "b" -font {symbol 12}
		    }
		}
	    }
	}
    }

    set panzoomdlg(clabel) $panzoom(system)
    switch -- $panzoom(system) {
	image -
	physical -
	amplifier -
	detector {}
	default {
	    if [$current(frame) has wcs $panzoom(system)] {
		if [$current(frame) has wcs equatorial $panzoom(system)] {
		    set panzoomdlg(clabel) $panzoom(sky)
		} else {
		    set name [$current(frame) get wcs name $panzoom(system)]
		    if {$name != {}} {
			set panzoomdlg(clabel) $name
		    }
		}
	    }
	}
    }
}

proc UpdateZoomDialog {} {
    global panzoom
    global panzoomdlg
    global current

    if {$current(frame) != {}} {
	global debug
	if {$debug(tcl,update)} {
	    puts "UpdateZoomDialog"
	}

	if {[winfo exist $panzoom(top)]} {
	    if {$current(frame) != {}} {
		set zz [$current(frame) get zoom]
		set panzoomdlg(zoom,x) [lindex $zz 0]
		set panzoomdlg(zoom,y) [lindex $zz 1]
		set panzoomdlg(rotate) [$current(frame) get rotate]
	    } else {
		set panzoomdlg(zoom,x) {}
		set panzoomdlg(zoom,y) {}
		set panzoomdlg(rotate) {}
	    }

	    PanZoomDialogCoord
	    UpdateAlignWCSMenu $panzoom(mb).zoom
	}
    }
}

proc ProcessPanCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    switch -- [string tolower [lindex $var $i]] {
	to {
	    set sys [lindex $var [expr $i+3]]
	    set sky [lindex $var [expr $i+4]]
	    switch -- $sys {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			wcs $sys
		    incr i 3
		}
		wcs -
		wcsa -
		wcsb -
		wcsc -
		wcsd -
		wcse -
		wcsf -
		wcsg -
		wcsh -
		wcsi -
		wcsj -
		wcsk -
		wcsl -
		wcsm -
		wcsn -
		wcso -
		wcsp -
		wcsq -
		wcsr -
		wcss -
		wcst -
		wcsu -
		wcsv -
		wcsw -
		wcsx -
		wcsy -
		wcsz {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			$sys $sky
		    incr i 4
		}
		image -
		physical -
		amplifier -
		detector {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			$sys {}
		    incr i 3
		}
		{} -
		default {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			physical {}
		    incr i 2
		}
	    }
	}
	default {
	    set sys [lindex $var [expr $i+2]]
	    set sky [lindex $var [expr $i+3]]
	    switch -- $sys {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			wcs $sys
		    incr i 2
		}
		wcs -
		wcsa -
		wcsb -
		wcsc -
		wcsd -
		wcse -
		wcsf -
		wcsg -
		wcsh -
		wcsi -
		wcsj -
		wcsk -
		wcsl -
		wcsm -
		wcsn -
		wcso -
		wcsp -
		wcsq -
		wcsr -
		wcss -
		wcst -
		wcsu -
		wcsv -
		wcsw -
		wcsx -
		wcsy -
		wcsz {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			$sys $sky
		    incr i 3
		}
		image -
		physical -
		amplifier -
		detector {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			$sys {}
		    incr i 2
		}
		{} -
		default {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			physical {}
		    incr i 1
		}

	    }
	}
    }
}

proc ProcessZoomCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current

    switch -- [string tolower [lindex $var $i]] {
	to {
	    switch -- [string tolower [lindex $var [expr $i+1]]] {
		fit {
		    ZoomToFit
		    incr i
		}
		default {
		    set z1 [lindex $var [expr $i+1]]
		    set z2 [lindex $var [expr $i+2]]
		    if {[string is double $z2] && $z2 != {}} {
			set current(zoom) " $z1 $z2 "
			incr i 2
		    } else {
			set current(zoom) " $z1 $z1 "
			incr i
		    }
		    ChangeZoom
		}
	    }
	}
	default {
	    set z1 [lindex $var $i]
	    set z2 [lindex $var [expr $i+1]]
	    if {[string is double $z2] && $z2 != {}} {
		Zoom $z1 $z2
		incr i
	    } else {
		Zoom $z1 $z1
	    }
	}
    }
}

proc ProcessOrientCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    set current(orient) [string tolower [lindex $var $i]]
    ChangeOrient
}

proc ProcessRotateCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    switch -- [string tolower [lindex $var $i]] {
	to {
	    set current(rotate) [lindex $var [expr $i+1]]
	    ChangeRotate
	    incr i
	}
	default {Rotate [lindex $var $i]}
    }
}
